@@ -282,7 +282,7 @@ SolvAddDebugInfo(
     dwError = testcase_write(pSolv, pszDir, dwResultFlags, NULL, NULL);
     if(dwError == 0)
     {
-        fprintf(stderr, "Could not write debugdata to folder %s\n", pszDir);
+        pr_err("Could not write debugdata to folder %s\n", pszDir);
     }
     //need not fail if debugdata write fails.
     dwError = 0;
@@ -315,7 +315,7 @@ TDNFGoal(
     Id  dwId = 0;
     int nProblems = 0;
     char** ppszExcludes = NULL;
-    uint32_t dwCount = 0;
+    uint32_t dwExcludeCount = 0;
 
     if(!pTdnf || !ppInfo || !pQueuePkgList)
     {
@@ -323,6 +323,9 @@ TDNFGoal(
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
+    dwError = TDNFPkgsToExclude(pTdnf, &dwExcludeCount, &ppszExcludes);
+    BAIL_ON_TDNF_ERROR(dwError);
+
     queue_init(&queueJobs);
     if (nAlterType == ALTER_UPGRADEALL)
     {
@@ -345,7 +348,8 @@ TDNFGoal(
         for (i = 0; i < pQueuePkgList->count; i++)
         {
             dwId = pQueuePkgList->elements[i];
-            TDNFAddGoal(pTdnf, nAlterType, &queueJobs, dwId);
+            TDNFAddGoal(pTdnf, nAlterType, &queueJobs, dwId,
+                        dwExcludeCount, ppszExcludes);
         }
     }
 
@@ -356,21 +360,15 @@ TDNFGoal(
     dwError = SolvAddFlagsToJobs(&queueJobs, nFlags);
     BAIL_ON_TDNF_ERROR(dwError);
 
-    if (nAlterType == ALTER_UPGRADEALL)
+    if (dwExcludeCount != 0 && ppszExcludes)
     {
-        dwError = TDNFPkgsToExclude(pTdnf, &dwCount, &ppszExcludes);
-        BAIL_ON_TDNF_ERROR(dwError);
-
-        if (dwCount != 0 && ppszExcludes)
+        if (!pTdnf->pSack || !pTdnf->pSack->pPool)
         {
-            if (!pTdnf->pSack || !pTdnf->pSack->pPool)
-            {
-                dwError = ERROR_TDNF_INVALID_PARAMETER;
-                BAIL_ON_TDNF_ERROR(dwError);
-            }
-            dwError = SolvAddExcludes(pTdnf->pSack->pPool, ppszExcludes);
+            dwError = ERROR_TDNF_INVALID_PARAMETER;
             BAIL_ON_TDNF_ERROR(dwError);
         }
+        dwError = SolvAddExcludes(pTdnf->pSack->pPool, ppszExcludes);
+        BAIL_ON_TDNF_ERROR(dwError);
     }
 
     pSolv = solver_create(pTdnf->pSack->pPool);
@@ -400,6 +398,12 @@ TDNFGoal(
         dwError = TDNFGetSkipProblemOption(pTdnf, &dwSkipProblem);
         BAIL_ON_TDNF_ERROR(dwError);
 
+        if (nAlterType == ALTER_UPGRADE && dwExcludeCount != 0 && ppszExcludes)
+        {
+            /* if we had packages to exclude, then we'd have diabled ones too */
+            dwSkipProblem |= SKIPPROBLEM_DISABLED;
+        }
+
         dwError = SolvReportProblems(pTdnf->pSack, pSolv, dwSkipProblem);
         BAIL_ON_TDNF_ERROR(dwError);
     }
@@ -455,16 +459,15 @@ TDNFAddGoal(
     PTDNF pTdnf,
     TDNF_ALTERTYPE nAlterType,
     Queue* pQueueJobs,
-    Id dwId
+    Id dwId,
+    uint32_t dwCount,
+    char** ppszExcludes
     )
 {
     uint32_t dwError = 0;
     char* pszPkg = NULL;
-    char** ppszExcludes = NULL;
     char** ppszPackagesTemp = NULL;
     char* pszName = NULL;
-    uint32_t dwCount = 0;
-    Queue queueJob = {0};
 
     if(!pQueueJobs || dwId == 0 || !pTdnf->pSack || !pTdnf->pSack->pPool)
     {
@@ -472,34 +475,32 @@ TDNFAddGoal(
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
-    queue_init(&queueJob);
-    if (nAlterType == ALTER_UPGRADE)
+    if (dwCount != 0 && ppszExcludes)
     {
-        dwError = TDNFPkgsToExclude(pTdnf, &dwCount, &ppszExcludes);
+        dwError = SolvGetPkgNameFromId(
+                      pTdnf->pSack,
+                      dwId,
+                      &pszName);
         BAIL_ON_TDNF_ERROR(dwError);
-        if (dwCount != 0 && ppszExcludes)
+        ppszPackagesTemp = ppszExcludes;
+
+        while(ppszPackagesTemp && *ppszPackagesTemp)
         {
-            dwError = SolvGetPkgNameFromId(
-                          pTdnf->pSack,
-                          dwId,
-                          &pszName);
-            BAIL_ON_TDNF_ERROR(dwError);
-            ppszPackagesTemp = ppszExcludes;
-            while(ppszPackagesTemp && *ppszPackagesTemp)
+            if (SolvIsGlob(*ppszPackagesTemp))
             {
-               if (SolvIsGlob(*ppszPackagesTemp))
-               {
-                   if (!fnmatch(*ppszPackagesTemp, pszName, 0))
-                      goto cleanup;
-               }
-               else if (!strcmp(pszName, *ppszPackagesTemp))
-               {
-                   goto cleanup;
-               }
-               ++ppszPackagesTemp;
+                if (!fnmatch(*ppszPackagesTemp, pszName, 0))
+                {
+                    goto cleanup;
+                }
             }
+            else if (!strcmp(pszName, *ppszPackagesTemp))
+            {
+                goto cleanup;
+            }
+            ++ppszPackagesTemp;
         }
     }
+
     switch(nAlterType)
     {
         case ALTER_DOWNGRADEALL:
@@ -527,9 +528,7 @@ TDNFAddGoal(
     }
 cleanup:
     TDNF_SAFE_FREE_MEMORY(pszPkg);
-    TDNF_SAFE_FREE_STRINGARRAY(ppszExcludes);
     TDNF_SAFE_FREE_MEMORY(pszName);
-    queue_free(&queueJob);
     return dwError;
 
 error:
@@ -620,4 +619,3 @@ error:
     }
     goto cleanup;
 }
-
